/*
 Copyright (C) 2004  Eddie Kelley  <eddie@kelleycomputing.net>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#import "SpotlightIndexerController.h"

@implementation SpotlightIndexerController

-(void) awakeFromNib
{
	indexRunning = FALSE;
	importInput = FALSE;
	importOutput = FALSE;
	importTerse = FALSE;
	importVerbose = FALSE;
	importWarning = FALSE;
	pluginLoading = FALSE;
	timing = FALSE;
	forceImport = TRUE;
	wait = FALSE;
	setAttributes = FALSE;
	indexArray = [self indexArray];
	
	// We tell the table that we'll be dragging in filenames
    [indexTableView registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,nil]];

	[self setWaitInterval:[NSNumber numberWithInt:0]];
	[self setLogLevel:[NSNumber numberWithInt:0]];
	userDefaults = [NSUserDefaults standardUserDefaults];
	[self updatePreferencesPanel];
	[indexTableView reloadData];
}

// callback by other applications (Finder) telling us to openFiles:file
- (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename
{ 
	NSMutableArray *filenames = [NSMutableArray arrayWithCapacity:1];
	
	// if the task is idle, add dropped file/folder to indexArray
	if(![indexTask isRunning]){
		[filenames addObject:filename];
		[self appendArrayToIndexArray:filenames];
		return TRUE;
	}
	// otherwise, continue import
	else
	{
		return FALSE;
	}
}

// If the user attempts to close the window, 
-(BOOL)windowShouldClose:(id)sender
{
	// quit when idle
	if(![indexTask isRunning]){
		[NSApp terminate:self];
		return YES;
	}
	else
		return NO;
}

- (NSNumber *) logLevel
{
	if([userDefaults objectForKey:@"logLevel"] != nil)
	{
		logLevel = [userDefaults objectForKey:@"logLevel"];
	}
	else{
		logLevel = [NSNumber numberWithInt:0];
		[userDefaults setObject:logLevel forKey:@"logLevel"];
	}
	return logLevel;
}

- (void) setLogLevel:(NSNumber*)newLogLevel
{
	[userDefaults setObject:newLogLevel forKey:@"logLevel"];
	[userDefaults synchronize];
}

- (NSNumber *) waitInterval
{
	if([userDefaults objectForKey:@"waitInterval"] != nil)
	{
		waitInterval = [userDefaults objectForKey:@"waitInterval"];
	}
	else{
		waitInterval = [NSNumber numberWithInt:0];
		[userDefaults setObject:waitInterval forKey:@"waitInterval"];
		[userDefaults synchronize];
	}
	return waitInterval;
}

- (void) setWaitInterval:(NSNumber*)interval
{
	[userDefaults setObject:interval forKey:@"waitInterval"];
	[userDefaults synchronize];
}

- (NSMutableArray*) indexArray
{
	if([userDefaults objectForKey:@"indexArray"] != nil)
	{
		indexArray = [NSMutableArray arrayWithArray:[userDefaults objectForKey:@"indexArray"]];
	}
	else
		indexArray = nil;
	
	return indexArray;
}

-(void) setIndexArray:(NSMutableArray*)newIndexArray
{
	[userDefaults setObject:newIndexArray forKey:@"indexArray"];
}

-(void) appendArrayToIndexArray:(NSArray*)items
{
	NSMutableArray * newIndexArray = [NSMutableArray arrayWithCapacity:1];
	int i = 0;
	
	// read current array
	[newIndexArray addObjectsFromArray:[self indexArray]];
	
	// Insert the filenames into our array
	for (i=0;i<[items count];i++)
	{
		if(![[self indexArray] containsObject:[items objectAtIndex:i]])
			[newIndexArray addObject:[items objectAtIndex:i]];
	}
	
	//set preferences
	[userDefaults setObject:newIndexArray forKey:@"indexArray"];
	[userDefaults synchronize];
	[indexTableView reloadData];
}

- (void) removeItemsFromIndexArray:(NSArray*)items
{
	if([items count] > 0)
	{
		NSMutableArray * newIndexArray = [NSMutableArray arrayWithCapacity:1];
		int i = 0;
		
		// read current array
		[newIndexArray addObjectsFromArray:[self indexArray]];
		
		if([items count] == 1)
		{
			// remove a single filename from our array
			[newIndexArray removeObjectIdenticalTo:[items objectAtIndex:i]];
		}
		else
		{
			// remove multiple filenames from our array
			while(i < [items count])
			{
				if([newIndexArray containsObject:[items objectAtIndex:i]]){
					[newIndexArray removeObjectIdenticalTo:[items objectAtIndex:i]];
					i++;
					
				}
				else
				{
					i = [items count];
			
				}
			}
		}
		
	//set preferences
		[userDefaults setObject:newIndexArray forKey:@"indexArray"];
		[userDefaults synchronize];
		[indexTableView reloadData];
	}
	
}

#pragma mark Preferences

-(IBAction) beginPreferencesPanel:(id)sender
{
	[NSApp beginSheet:Preferences
	   modalForWindow:[NSApp mainWindow]
		modalDelegate:self
	   didEndSelector:NULL
		  contextInfo:nil];
}

-(void) endPreferencesPanel
{
	[Preferences orderOut:self];
    [NSApp endSheet:Preferences];
}

-(void) updatePreferencesPanel
{
	// read, and set logLevel 
	if([[self logLevel] intValue] != 0)
	{
		[loggingMatrix selectCellWithTag:[[self logLevel] intValue]];
		[importInputButton setEnabled:TRUE];
		[importOutputButton setEnabled:TRUE];
		[importTerseButton setEnabled:TRUE];
		[importVerboseButton setEnabled:TRUE];
		[importWarningsButton setEnabled:TRUE];
		[pluginLoadingButton setEnabled:TRUE];
	}
	else
	{
		// if the we aren't logging, disable categories
		[self setLogLevel:[NSNumber numberWithInt:0]];
		[loggingMatrix selectCellWithTag:[[self logLevel] intValue]];
		[importInputButton setEnabled:FALSE];
		[importOutputButton setEnabled:FALSE];
		[importTerseButton setEnabled:FALSE];
		[importVerboseButton setEnabled:FALSE];
		[importWarningsButton setEnabled:FALSE];
		[pluginLoadingButton setEnabled:FALSE];
	}
	
	// read import intput button preferences
	if(importInput = [userDefaults boolForKey:@"importInput"])
		[importInputButton setState:importInput];
	else
		[userDefaults setBool:importInput forKey:@"importInput"];
	
	// read import output button preferences
	if(importOutput = [userDefaults boolForKey:@"importOutput"])
		[importOutputButton setState:importOutput];
	else
		[userDefaults setBool:importOutput forKey:@"importOutput"];
	
	// read import terse button preferences
	if(importTerse = [userDefaults boolForKey:@"importTerse"])
		[importTerseButton setState:importTerse];
	else
		[userDefaults setBool:importTerse forKey:@"importTerse"];
	
	// read import verbose button preferences
	if(importVerbose = [userDefaults boolForKey:@"importVerbose"])
		[importVerboseButton setState:importVerbose];
	else
		[userDefaults setBool:importVerbose forKey:@"importVerbose"];
	
	// read import warning button preferences
	if(importWarning = [userDefaults boolForKey:@"importWarning"])
		[importWarningsButton setState:importWarning];
	else
		[userDefaults setBool:importWarning forKey:@"importWarning"];
	
	// read plugin loading button preferences
	if(pluginLoading = [userDefaults boolForKey:@"pluginLoading"])
		[pluginLoadingButton setState:pluginLoading];
	else
		[userDefaults setBool:pluginLoading forKey:@"pluginLoading"];
	
		// read timing button preferences
	if(timing = [userDefaults boolForKey:@"timing"])
		[timingButton setState:timing];
	else
		[userDefaults setBool:timing forKey:@"timing"];
	
	// read forceImport button preferences
	if(forceImport = [userDefaults boolForKey:@"forceImport"])
		[forceImportButton setState:forceImport];
	else
		[userDefaults setBool:forceImport forKey:@"forceImport"];
	
	// read wait button preferences
	if(wait = [userDefaults boolForKey:@"wait"])
		[waitButton setState:wait];
	else
		[userDefaults setBool:wait forKey:@"wait"];	
	
	// read setAttributes preferences
	if(setAttributes = [userDefaults boolForKey:@"setAttributes"])
		[attributesButton setState:setAttributes];
	else
		[userDefaults setBool:setAttributes forKey:@"setAttributes"];	
	
	if(wait)
	{
		[waitTextField setDoubleValue:[[self waitInterval] doubleValue]];
		[waitTextField setEnabled:YES];
	}
	else
	{
		[waitTextField setDoubleValue:[[self waitInterval] doubleValue]];
		[waitTextField setEnabled:NO];
	}
}

# pragma mark IBActions

- (IBAction)addButtonAction:(id)sender
{
	NSMutableArray *newIndexArray = [NSMutableArray arrayWithCapacity:1], *filenames = [NSMutableArray arrayWithCapacity:1];
	[newIndexArray addObjectsFromArray:[self indexArray]];
	
	openPanel = [NSOpenPanel openPanel];
	[openPanel setCanChooseDirectories:TRUE];
	[openPanel setAllowsMultipleSelection:TRUE];
	//[openPanel setAccessoryView:AddView];
	
	if([openPanel runModal] == NSOKButton)
	{
		[filenames addObjectsFromArray:[openPanel filenames]];
		if([filenames count] > 0)
		{
			[self appendArrayToIndexArray:filenames];
		}
	}
	
}

- (IBAction)removeButtonAction:(id)sender
{
	if(([indexTableView selectedRow] >= 0) && ([indexTableView numberOfSelectedRows] == 1)){
		[self removeItemsFromIndexArray:[NSArray arrayWithObjects:[[self indexArray] objectAtIndex:[indexTableView selectedRow]], nil]];
	}
	else if(([indexTableView selectedRow] >= 0) && ([[self indexArray] count] > 0)){ 
		int i = 0;
		NSMutableArray *removeList = [NSMutableArray arrayWithCapacity:1]; 
		
		for (i=0;i<=[[self indexArray] count];i++)
			if([indexTableView isRowSelected:i])
				[removeList addObject:[[self indexArray] objectAtIndex:i]];
		[self removeItemsFromIndexArray:removeList];
	}
	if([[self indexArray] count] > 0)
		[indexTableView selectRow:0 byExtendingSelection:FALSE];
}

-(IBAction) addTextFieldAction:(id)sender
{
	if([fileManager fileExistsAtPath:[sender stringValue]])
		[openPanel setDirectory:[sender stringValue]];
	else
		NSBeep();
}

- (IBAction)indexButtonAction:(id)sender
{
	if(!indexRunning)
	{
		NSMutableArray *arguments = [NSMutableArray arrayWithCapacity:1];
		
	// forceImport preference
		if(forceImport)
			[arguments addObject:@"-f"];
		
	// display timing if user has chosen
		if(timing)
			[arguments addObject:@"-V"];
		
	// debugging (logging) information
		if([[self logLevel] intValue] != 0)
		{
		//log level greater than 0 
			[arguments addObject:@"-d"];
			
			[arguments addObject:[[self logLevel] stringValue]];
			
			if(importInput)
				[arguments addObject:@"import-input"];
			if(importOutput)
				[arguments addObject:@"import-output"];
			if(importTerse)
				[arguments addObject:@"import-terse"];
			if(importVerbose)
				[arguments addObject:@"import-verbose"];
			if(importWarning)
				[arguments addObject:@"import-warning"];
			if(pluginLoading)
				[arguments addObject:@"plugin-loading"];
		}
		
	// display timing if user has chosen
		if(setAttributes)
			[arguments addObject:@"-n"];
		
		if(wait && ([[self waitInterval] doubleValue] != 0))
		{
			[arguments addObject:@"-w"];
			[arguments addObject:[[self waitInterval] stringValue]];
		}
		
	// add file/folder name
		[arguments addObjectsFromArray:[self indexArray]];
		
	// launch the task
		[self openTask:@"/usr/bin/mdimport" withArguments:arguments];
		
	}
	else
	{
	// stop the indexTask
		[indexTask stopProcess];
	}
}


- (IBAction)logMatrixAction:(id)sender
{
	int selection = [[[sender selectedCell] title] intValue];
	
	[self setLogLevel:[NSNumber numberWithInt:selection]];
	if([[[sender selectedCell] title] isEqual:@"0"])
	{
		[importTerseButton setEnabled:FALSE];
		[importInputButton setEnabled:FALSE];
		[importOutputButton setEnabled:FALSE];
		[importWarningsButton setEnabled:FALSE];
		[importVerboseButton setEnabled:FALSE];
		[pluginLoadingButton setEnabled:FALSE];
	}
	[self updatePreferencesPanel];
}

- (IBAction)importTerseButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		importTerse = TRUE;
	}
	else
	{
		importTerse = FALSE;
	}
	[userDefaults setBool:importTerse forKey:@"importTerse"];
}

- (IBAction)importInputButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		importInput = TRUE;
	}
	else
	{
		importInput = FALSE;
	}
	[userDefaults setBool:importInput forKey:@"importInput"];
}

- (IBAction)importOutputButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		importOutput = TRUE;
	}
	else
	{
		importOutput = FALSE;
	}
	[userDefaults setBool:importOutput forKey:@"importOutput"];
}

- (IBAction)importWarningsButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		importWarning = TRUE;
	}
	else
	{
		importWarning = FALSE;
	}
	[userDefaults setBool:importWarning forKey:@"importWarning"];
}

- (IBAction)importVerboseButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		importVerbose = TRUE;
	}
	else
	{
		importVerbose = FALSE;
	}
	[userDefaults setBool:importVerbose forKey:@"importVerbose"];
}

- (IBAction)pluginLoadingButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		pluginLoading = TRUE;
	}
	else
	{
		pluginLoading = FALSE;
	}
	[userDefaults setBool:pluginLoading forKey:@"pluginLoading"];
}

- (IBAction)timingButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		timing = TRUE;
	}
	else
	{
		timing = FALSE;
	}
	[userDefaults setBool:timing forKey:@"timing"];
}

- (IBAction)forceImportButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		timing = TRUE;
	}
	else
	{
		timing = FALSE;
	}
	[userDefaults setBool:timing forKey:@"forceImport"];
}

- (IBAction)okButtonAction:(id)sender
{
	[userDefaults synchronize];
	[self endPreferencesPanel];
}

- (IBAction)showHideLogAction:(id)sender
{
	[LogDrawer setPreferredEdge:NSMinYEdge];
	if([LogDrawer state] == NSDrawerOpenState)
		[LogDrawer close];
	else
		[LogDrawer openOnEdge:NSMinYEdge];
}

- (IBAction)attributesButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		setAttributes = TRUE;
	}
	else
	{
		setAttributes = FALSE;
	}
	[userDefaults setBool:setAttributes forKey:@"setAttributes"];
}

- (IBAction)waitButtonAction:(id)sender
{
	if([sender state] == 1)
	{
		[waitTextField setEnabled:TRUE];
		wait = TRUE;
	}
	else
	{
		wait = FALSE;
		[waitTextField setEnabled:FALSE];
	}
	[userDefaults setBool:wait forKey:@"wait"];
}

- (IBAction) waitTextFieldAction:(id)sender
{
	[userDefaults setObject:[NSNumber numberWithDouble:[sender doubleValue]] forKey:@"waitInterval"];
}

#pragma mark Options Menu Actions

- (IBAction)printAttributesAction:(id)sender
{
	if(!([LogDrawer state] == NSDrawerOpenState))
		[LogDrawer openOnEdge:NSMinYEdge];
	if(![indexTask isRunning])
		[self openTask:@"/usr/bin/mdimport" withArguments:[NSArray arrayWithObjects:@"-A", nil]];
}

- (IBAction)printImportersAction:(id)sender
{
	if(!([LogDrawer state] == NSDrawerOpenState))
		[LogDrawer openOnEdge:NSMinYEdge];
	if(![indexTask isRunning])
		[self openTask:@"/usr/bin/mdimport" withArguments:[NSArray arrayWithObjects:@"-L", nil]];
}

- (IBAction)printSchemaAction:(id)sender
{
	if(!([LogDrawer state] == NSDrawerOpenState))
		[LogDrawer openOnEdge:NSMinYEdge];
	if(![indexTask isRunning])
		[self openTask:@"/usr/bin/mdimport" withArguments:[NSArray arrayWithObjects:@"-X", nil]];
}

- (IBAction)reimportAction:(id)sender
{
	openPanel = [NSOpenPanel openPanel];
	if([openPanel runModalForDirectory:@"/System/Library/Spotlight" 
								  file:nil types:[NSArray arrayWithObjects:@"mdimporter", nil]] == NSOKButton)
	{
		if(![indexTask isRunning])
			[self openTask:@"/usr/bin/mdimport" withArguments:[NSArray arrayWithObjects:@"-r", [openPanel filename], nil]];

	}
	else
	{}
		
}

#pragma mark TableView controls

-(int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [[self indexArray] count];
}

// Stop the table's rows from being editable when we double-click on them
- (BOOL)tableView:(NSTableView *)tableView shouldEditTableColumn:(NSTableColumn *)tableColumn row:(int)row
{  
    return FALSE;
}

- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
            row:(int)rowIndex
{
	NSString  *ident;
    NSObject  *object;
    NSString  *path;

    ident = [aTableColumn identifier];
	
	if([ident isEqualToString:@"path"] && (rowIndex < [[self indexArray] count]))
	{
		// return volume name
		path = [[self indexArray] objectAtIndex:rowIndex];
		object = [path copyWithZone:nil];
		[path release];
	}
	else if (rowIndex < [[self indexArray] count]){
		// return path item's icon
		path = [[self indexArray] objectAtIndex:rowIndex];
		object = [[NSWorkspace sharedWorkspace] iconForFile: path];
    }
	else
		object = nil;
	
	return object;
}

- (void)tableView:(NSTableView *)aTableView	setObjectValue:anObject
   forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    [[self indexArray] insertObject:anObject atIndex:rowIndex];
    return;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
		
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
    [tv setDropRow:[tv numberOfRows] dropOperation:NSTableViewDropAbove];
    return NSTableViewDropAbove;
}

// This routine does the actual processing for a drag-and-drop operation on a tableview.
// As the tableview's data source, we get this call when it's time to update our backend data.
- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
    // Get the drag-n-drop pasteboard
    NSPasteboard *myPasteboard=[info draggingPasteboard];
    // What type of data are we going to allow to be dragged?  The pasteboard
    // might contain different formats we use only filenames
    NSArray *typeArray=[NSArray arrayWithObjects:NSFilenamesPboardType,nil];
    NSString *availableType;
    NSArray *filesList;
    
    // find the best match of the types we'll accept and what's actually on the pasteboard
    availableType=[myPasteboard availableTypeFromArray:typeArray];
    
	// In the file format type that we're working with, get all data on the pasteboard
    filesList=[myPasteboard propertyListForType:availableType];
    
	[self appendArrayToIndexArray:filesList];
	
    // Select the last item.
    [indexTableView selectRow:[[self indexArray] count] byExtendingSelection:NO];
    return YES;
}


#pragma mark Task Control


// callback from taskwrapper with output
- (void)appendOutput:(NSString *)output
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// append program output to log view
	if(output != nil)
		[[logTextView textStorage] appendAttributedString: [[[NSAttributedString alloc]
								initWithString: output] autorelease]];
	
	
	// setup a selector to be called the next time through the event loop to scroll
	// the view to the just pasted text.  We don't want to scroll right now,
	// because of a bug in Mac OS X version 10.1 that causes scrolling in the context
	// of a text storage update to starve the app of events
	//  [self performSelector:@selector(scrollToVisible:) withObject:nil afterDelay:0.0];
	
    [logTextView scrollRangeToVisible:NSMakeRange([[logTextView string] length], 0)];
	[pool release];
}

// This routine is called after adding new results to the text view's backing store.
// We now need to scroll the NSScrollView in which the NSTextView sits to the part
// that we just added at the end
- (void)scrollToVisible:(id)ignore {
    [logTextView scrollRangeToVisible:NSMakeRange([[logTextView string] length], 0)];
}

// This method is a callback which your controller can use to do other initialization when a process
// is launched.
- (void)processStarted
{
	indexRunning = TRUE;
	[indexButton setTitle:NSLocalizedString(@"Cancel", @"Cancel button value")];
	[addButton setEnabled:FALSE];
	[removeButton setEnabled:FALSE];
	[optionsMenu setEnabled:FALSE];
	
	[indexProgress setUsesThreadedAnimation:TRUE];
	[indexProgress startAnimation:self];
	
	// clear the log view
	[[logTextView textStorage] setAttributedString: [[[NSAttributedString alloc]
								initWithString:@""] autorelease]];
}

// This method is a callback which your controller can use to do other cleanup when a process
// is halted.
- (void)processFinished
{
	indexRunning = FALSE;
	[indexButton setTitle:NSLocalizedString(@"Index", @"Index button string")];
	[addButton setEnabled:TRUE];
	[removeButton setEnabled:TRUE];
	[optionsMenu setEnabled:TRUE];
	
	[indexProgress stopAnimation:self];
	
}

-(int) openTask:(NSString*)path withArguments:(NSArray*)arguments
{
	// Variables
	int processID = 0;
	NSMutableArray *args = [NSMutableArray arrayWithCapacity:1];
	
	// Set launch arguments
	[args addObject:path];
	[args addObjectsFromArray:arguments];
	
	// If the task is already running, release
	if (indexRunning!=nil)
        [indexTask release];
	// Let's allocate memory for and initialize a new TaskWrapper object, passing
	// in ourselves as the controller for this TaskWrapper object, the path
	// to the command-line tool, and the contents of the text field that 
	// displays what the user wants to search on
	indexTask=[[TaskWrapper alloc] initWithController:self arguments:args];
	// kick off the process asynchronously
	[indexTask startProcess];
	processID = [indexTask processID];
	return processID;
}


@end
