/* SpotlightIndexerController 

Copyright (C) 2004  Eddie Kelley  <eddie@kelleycomputing.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Version history: (most recent first)

	v. 0.2.2b		3/18/2007
		 Universal binary
		 "Force Import" is now the default preference

	v. 0.2.1b		8/21/2005
		 Now uses table view for indexing of multiple folders
			 Add items by drag and drop
			 Add with "Open" menu option
			 Add with Dock icon drop
		 List items are saved in user defaults
	
	v. 0.2b			8/11/2005
		 Added preference for wait interval between scans
		 Added debugging option to avoid setting attributes on files when scanning
		 Added Localization (French, Dutch, German, Spanish, Italian)
		 Added "Options" menu
			- Added "Print attributes" option
			- Added "Print schema file" option
			- Added "Print list of importers" option
			- Added "Re-index for plugin..." option

	v.0.1.1d2 (0.1.1b)	8/1/2005	Minor dev version change
		 Indexes items dropped on dock icon
		 User can now selectively force import
	
	v.0.1.1d1	7/26/2005	Minor dev version change
		 Lowered CPU usage when idle
		 Corrected logging issues (when very verbose)

	v.0.1b	6/1/2005	Initial beta release
*/


#import <Cocoa/Cocoa.h>
#import <TaskWrapper.h>

@interface SpotlightIndexerController : NSObject <TaskWrapperController>
{
    IBOutlet id indexButton;
    IBOutlet id indexProgress;
    IBOutlet id indexTextField;
	
	// log items
    IBOutlet id LogDrawer;
	IBOutlet id LogView;
    IBOutlet id logTextView;
	
	// preferences items
	IBOutlet id attributesButton;
	IBOutlet id forceImportButton;
    IBOutlet id Preferences;
	IBOutlet id loggingMatrix;
	IBOutlet id importInputButton;
	IBOutlet id importOutputButton;
	IBOutlet id importTerseButton;
	IBOutlet id importVerboseButton;
	IBOutlet id importWarningsButton;
	IBOutlet id pluginLoadingButton;
    IBOutlet id timingButton;
	IBOutlet id okButton;
	IBOutlet id waitButton;
	IBOutlet id waitTextField;
	IBOutlet id optionsMenu;
	IBOutlet id addButton;
	IBOutlet id removeButton;
	IBOutlet id indexTableView;
	IBOutlet id AddView;
	IBOutlet id addTextField;
	
	TaskWrapper *indexTask;
	BOOL indexRunning;
	BOOL importInput;
	BOOL importOutput;
	BOOL importTerse;
	BOOL importVerbose;
	BOOL importWarning;
	BOOL pluginLoading;
	BOOL timing;
	BOOL forceImport;
	BOOL wait;
	BOOL setAttributes;
	
	NSNumber *logLevel, *waitInterval;
	NSFileManager *fileManager;
	NSUserDefaults * userDefaults;
	NSMutableArray *indexArray, *enabledArray;
}


NSOpenPanel *openPanel;

// Methods
- (NSNumber *)logLevel;
- (void) setLogLevel:(NSNumber*)newLogLevel;
- (NSNumber *) waitInterval;
- (void) setWaitInterval:(NSNumber*)interval;
- (void) updatePreferencesPanel;
- (NSMutableArray*) indexArray;
- (NSMutableArray*) enabledArray;
- (void) setIndexArray:(NSMutableArray*)newIndexArray;
- (void) appendArrayToIndexArray:(NSArray*)items;

// Main UI Actions
- (IBAction)addButtonAction:(id)sender;
- (IBAction)removeButtonAction:(id)sender;
- (IBAction)indexButtonAction:(id)sender;
- (BOOL) windowShouldClose:(id)sender;

// Preferences actions
- (IBAction)attributesButtonAction:(id)sender;
- (IBAction)logMatrixAction:(id)sender;
- (IBAction)importTerseButtonAction:(id)sender;
- (IBAction)importInputButtonAction:(id)sender;
- (IBAction)importOutputButtonAction:(id)sender;
- (IBAction)importWarningsButtonAction:(id)sender;
- (IBAction)importVerboseButtonAction:(id)sender;
- (IBAction)pluginLoadingButtonAction:(id)sender;
- (IBAction)timingButtonAction:(id)sender;
- (IBAction)forceImportButtonAction:(id)sender;
- (IBAction)okButtonAction:(id)sender;
- (IBAction)waitButtonAction:(id)sender;

// Menu actions
- (IBAction)showHideLogAction:(id)sender;
- (IBAction)printAttributesAction:(id)sender;
- (IBAction)printImportersAction:(id)sender;
- (IBAction)printSchemaAction:(id)sender;
- (IBAction)reimportAction:(id)sender;

// Other actions
-(IBAction) beginPreferencesPanel:(id)sender;
-(IBAction) addTextFieldAction:(id)sender;

// Task control methods

-(int) openTask:(NSString*)path withArguments:(NSArray*)arguments;
	// Tthis method will be called when output arrives from the NSTask.
	// Output will come from both stdout and stderr, per the TaskWrapper implementation.
- (void)appendOutput:(NSString *)output;

	// This method is a callback which your controller can use to do other initialization when a process
	// is launched.
- (void)processStarted;

	// This method is a callback which your controller can use to do other cleanup when a process
	// is halted.
- (void)processFinished;

@end
